#ifndef __TCPServerSocket_h__
#define __TCPServerSocket_h__

#include "Socket.h"
#include "TCPSocket.h"
#include "Service.h"
#include "IPAddressList.h"

namespace core {
   
   ///
   /// TCPServerSocket
   ///
   /// Manage a socket connection as a TCP server type. Connections to the socket are processed through
   /// the accept functionality. 
   ///
   /// A list of connections is maintained in a vector object. 
   ///
   /// This object extends the BMACommand object as well so it can be added to a Console object and
   /// process commands to display status information.
   ///
   
   class TCPServerSocket : public TCPSocket, public Command {
      
    public:
      
      ///
      /// The constructor for the BMATCPSocket object.
      ///
      /// @param ePoll the EPoll instance that manages the socket.
      /// @param url the IP address for the socket to receive connection requests.
      /// @param port the port number that the socket will listen on.
      /// @param commandName the name of the command used to invoke the status display for this object.
      /// @return the instance of the BMATCPServerSocket.
      ///
      
      TCPServerSocket(EPoll &ePoll, Service &service, IPAddress address);
      
      ///
      /// The destructor for this object.
      ///
      
      ~TCPServerSocket();

      /// 
      /// The service object reference used to access associated server information such as sessions 
      /// and other extended managers.
      ///
      
      Service &service;
      
      ///
      /// If not NULL the blacklist object can be assigned to this server socket and the server
      /// IP addresses connecting to the server attempting to accept a socket are contained in 
      /// this list then the connection is rejected and no accept is granted. 
      ///
      
      IPAddressList *blackList;      
      ///
      /// If not NULL the blacklist object can be assigned to this server socket and the server
      /// IP addresses connecting to the server attempting to accept a socket are contained in 
      /// this list then the connection is rejected and no accept is granted. 
      ///
      IPAddressList *whiteList;
      
    protected:
            
      ///
      /// Override the virtual dataReceived since for the server these
      /// are requests to accept the new connection socket.
      /// No data is to be read or written when this method is called. It is the response to 
      /// the fact that a new connection is coming into the system 
      ///
      /// @param data the pointer to the buffer containing the received data.
      /// @param length the length of the associated data buffer.
      ///
      
      void onDataReceived(std::string data) override;      
      
      ///
      /// This method is called when the Command associated with this object is requested
      /// because a user has typed in the associated command name on a command entry line. 
      ///
      /// @param the session object to write the output to.
      ///
      
      int processCommand(std::string command, Session *session, std::stringstream &data) override;
		      
    private:

      Session * accept();
      
   };
   
}

#endif
